<?php
/**
 * Common functions only loaded on AJAX requests.
 *
 * @since 3.0.0
 * @version 8.0.0
 */

// Exit if accessed directly.
defined( 'ABSPATH' ) || exit;

/**
 * Load the template loop for the current object.
 *
 * @since 3.0.0
 *
 * @return string Template loop for the specified object
 */
function bp_nouveau_ajax_object_template_loader() {
	if ( ! bp_is_post_request() ) {
		wp_send_json_error();
	}

	if ( empty( $_POST['object'] ) ) {
		wp_send_json_error();
	}

	$object = sanitize_title( $_POST['object'] );

	// Nonce check!
	if ( empty( $_POST['nonce'] ) || ! wp_verify_nonce( $_POST['nonce'], 'bp_nouveau_' . $object ) ) {
		wp_send_json_error();
	}

	$scope = '';
	if (isset($_POST['scope'])) {
		$scope = $_POST['scope'];
	}

	$template = '';
	if (isset($_POST['template'])) {
		$template = $_POST['template'];
	}

	$args = false;
	$activity_list_wrapper = '{{activity-li}}';
	if ('activity' === $object) {
		$activity_args = bp_parse_args(bp_ajax_querystring( 'activity' ));
		$activity_args['excluded_types'] = array('joined_group');	// do not show "a member has joined the group" activities
		$activity_args['pinned'] = 'exclude-pinned';
		$args = array('r' => $activity_args);

		if (bp_is_group()) {
			$group_id = bp_get_current_group_id();
			$group = groups_get_group($group_id);
			if (!empty($group)) {
				$args['group_id'] = $group_id;
				$args['referrer-attr'] = CommunityAllyGroupDisplay::generate_ajax_referrer_attr($group);
			}
			$activity_list_wrapper = '<ul class="activity-list item-list bp-list communityally-group-activity-list-' . $group_id . '">{{activity-li}}</ul>';
		}
	}
	$result = communityally_activity_template_loader($object, $scope, $template, $args);
	if (false === $result) {
		wp_send_json_error();
	}
	if (isset($result['contents'])) {
		$result['contents'] = str_replace('{{activity-li}}', $result['contents'], $activity_list_wrapper);
	}
	// Locate the object template.
	wp_send_json_success( $result );
}
