/* global BP_Nouveau, _, Backbone, plupload, mOxie */
/* @since 3.0.0 */
/* @version 8.0.0 */
window.wp = window.wp || {};
window.bp = window.bp || {};

( function( bp, $ ) {
	bp.Nouveau = bp.Nouveau || {};

	// Bail if not set
	if ( typeof bp.Nouveau.Activity === 'undefined' || typeof BP_Nouveau === 'undefined' ) {
		return;
	}

	_.extend( bp, _.pick( wp, 'Backbone', 'ajax', 'template' ) );

	bp.Models      = bp.Models || {};
	bp.Collections = bp.Collections || {};
	bp.Views       = bp.Views || {};
	bp.Uploader       = bp.Uploader || {};


	/** User file uploader ****************************************************************/
	
	function createNewCommunityAllyUploader(browseButtonId, collection, attachmentType) {
		var isIE = navigator.userAgent.indexOf('Trident/') !== -1 || navigator.userAgent.indexOf('MSIE ') !== -1;

		let params  = BP_Nouveau.activity.upload_settings,
			supports = {
				upload: params.browser.supported
			},
			supported = supports.upload;

		if ( ! supported ) {
			return;
		}

		// Make sure flash sends cookies (seems in IE it does without switching to urlstream mode).
		if ( ! isIE && 'flash' === plupload.predictRuntime( params.defaults ) &&
			( ! params.defaults.required_features || ! params.defaults.required_features.hasOwnProperty( 'send_binary_string' ) ) ) {

			params.defaults.required_features = params.defaults.required_features || {};
			params.defaults.required_features.send_binary_string = true;
		}

		params.defaults.browse_button = browseButtonId;
		if ('image' === attachmentType) {
			params.defaults.filters.mime_types = [{title: 'Images', extensions: 'jpg,gif,png,jpeg'}];
		} else {
			params.defaults.filters.mime_types = [];
		}
		params.defaults.multipart_params.action = 'communityally_activity_upload_file';
		let uploader = new plupload.Uploader( params.defaults );

		/**
		 * After the Uploader has been initialized, initialize some behaviors for the dropzone.
		 *
		 * @event Init
		 * @param {plupload.Uploader} uploader Uploader instance.
		 */
		uploader.bind( 'Init', function( uploader ) {
			if ( 'html4' === uploader.runtime ) {
				uploader.settings.multipart_params.html4 = true;
			}
		} );

		// See https://core.trac.wordpress.org/ticket/37039.
		uploader.bind( 'postinit', function( up ) {
			up.refresh();
		});

		// Init CommunityAlly Uploader.
		uploader.init();

		/**
		 * After files were filtered and added to the queue, create a model for each.
		 *
		 * @event FilesAdded
		 * @param {plupload.Uploader} uploader Uploader instance.
		 * @param {Array}             files    Array of file objects that were added to queue by the user.
		 */
		uploader.bind( 'FilesAdded', function( uploader, files ) {
			collection.addUploadFiles(files);
			uploader.refresh();
			uploader.start();
		} );

		/**
		 * Update each file item on progress.
		 *
		 * @event UploadProgress
		 * @param {plupload.Uploader} uploader Uploader instance.
		 * @param {Object}            file
		 */
		uploader.bind( 'UploadProgress', function( uploader, file ) {
			collection.trigger('attachment:updateProgress-' + file.id, file);
		} );

		/**
		 * After a file is successfully uploaded, update its model.
		 *
		 * @event FileUploaded
		 * @param {plupload.Uploader} uploader Uploader instance.
		 * @param {plupload.File}     file     File that was uploaded.
		 * @param {Object}            response Object with response properties.
		 * @return {mixed}
		 */
		uploader.bind( 'FileUploaded', function( uploader, file, response ) {
			collection.trigger('attachment:fileUploaded-' + file.id, file, response);
		} );
		return uploader;
	}
	
	/**
	 * [Activity description]
	 * @type {Object}
	 */
	bp.Nouveau.Activity.postForm = {
		start: function() {
			this.views           = new Backbone.Collection();
			this.ActivityObjects = new bp.Collections.ActivityObjects();
			this.buttons         = new Backbone.Collection();

			this.postFormView();
		},

		postFormView: function() {
			// Do not carry on if the main element is not available.
			if ( ! $( '#bp-nouveau-activity-form' ).length ) {
				return;
			}

			// Create the BuddyPress Uploader
			var postForm = new bp.Views.PostForm();

			// Add it to views
			this.views.add( { id: 'post_form', view: postForm } );

			// Display it
			postForm.inject( '#bp-nouveau-activity-form' );
		}
	};

	if ( typeof bp.View === 'undefined' ) {
		// Extend wp.Backbone.View with .prepare() and .inject()
		bp.View = bp.Backbone.View.extend( {
			inject: function( selector ) {
				this.render();
				$(selector).html( this.el );
				this.views.ready();
			},

			prepare: function() {
				if ( ! _.isUndefined( this.model ) && _.isFunction( this.model.toJSON ) ) {
					return this.model.toJSON();
				} else {
					return {};
				}
			}
		} );
	}

	/** Models ****************************************************************/

	// The image attachment to activity
	bp.Models.Attachment = Backbone.Model.extend( {
		defaults: {
			ordinal: 0,
			type:   'image',
			status: 'upload',
			url:   '',
			percent: 0
		},
		initialize: function(file, ordinal, attachmentType) {
			this.set('name', file.name);
			this.set('ordinal', ordinal);
			this.set('type', attachmentType);
		},
		updateProgress: function(file) {
			this.set('percent', file.percent);
			this.trigger('attachment:changed', this);
		},
		uploadComplete: function(file, response) {
			try {
				response = JSON.parse(response.response);
			} catch (e) {
				alert(e);
			}
			this.set('percent', 100);
			this.set('status', 'complete');
			this.set('name', response.data.name);
			this.set('path', response.data.path);
			this.set('url', response.data.url);
			this.trigger('attachment:changed', this);
		}
	} );

	// The Activity to post
	bp.Models.Activity = Backbone.Model.extend( {
		defaults: {
			user_id:   0,
			item_id:   0,
			object:   '',
			content:  ''
		},
		initialize: function() {
			this.attachments = new bp.Collections.Attachments(null, { attachmentType: 'image' });
			this.fileAttachments = new bp.Collections.Attachments(null, { attachmentType: 'file' });
		},
		isUploadCompleted: function() {
			return this.attachments.isUploadCompleted() && this.fileAttachments.isUploadCompleted();
		}
	} );

	// Object, the activity is attached to (group or blog or any other)
	bp.Models.ActivityObject = Backbone.Model.extend( {
		defaults: {
			id          : 0,
			name        : '',
			avatar_url  : '',
			object_type : 'group'
		}
	} );

	/** Collections ***********************************************************/

	bp.Collections.Attachments = Backbone.Collection.extend( {
		model: bp.Models.Attachment,

		initialize: function(model, options) {
			this.itemOrdinal = 0;
			this.attachmentType = options.attachmentType;
		},
		parse: function( resp ) {
			if ( ! _.isArray( resp ) ) {
				resp = [resp];
			}

			return resp;
		},

		addUploadFiles: function(files) {
			for (let i in files) {
				++this.itemOrdinal;
				let f = files[i],
					item = new bp.Models.Attachment(f, this.itemOrdinal, this.attachmentType);
				this.add(item);
				this.on('attachment:updateProgress-' + f.id, item.updateProgress, item);
				this.on('attachment:fileUploaded-' + f.id, item.uploadComplete, item);
			}
		},

		isUploadCompleted: function() {
			let completed = true;
			this.each(function(item) {
				let status = item.get('status');
				completed &= 'complete' === status;
			});
			return completed;
		}

	} );

	// Objects, the activity can be attached to (groups or blogs or any others)
	bp.Collections.ActivityObjects = Backbone.Collection.extend( {
		model: bp.Models.ActivityObject,

		sync: function( method, model, options ) {

			if ( 'read' === method ) {
				options = options || {};
				options.context = this;
				options.data = _.extend( options.data || {}, {
					action: 'bp_nouveau_get_activity_objects'
				} );

				return bp.ajax.send( options );
			}
		},

		parse: function( resp ) {
			if ( ! _.isArray( resp ) ) {
				resp = [resp];
			}

			return resp;
		}

	} );

	/** Views *****************************************************************/

	// Feedback messages
	bp.Views.activityFeedback = bp.View.extend( {
		tagName  : 'div',
		id       : 'message',
		template : bp.template( 'activity-post-form-feedback' ),

		initialize: function() {
			this.model = new Backbone.Model();

			if ( this.options.value ) {
				this.model.set( 'message', this.options.value, { silent: true } );
			}

			this.type  = 'info';

			if ( ! _.isUndefined( this.options.type ) && 'info' !== this.options.type ) {
				this.type = this.options.type;
			}

			this.el.className = 'bp-messages bp-feedback ' + this.type ;
		}
	} );

	// Regular input
	bp.Views.ActivityInput = bp.View.extend( {
		tagName  : 'input',

		attributes: {
			type : 'text'
		},

		initialize: function() {
			if ( ! _.isObject( this.options ) ) {
				return;
			}

			_.each( this.options, function( value, key ) {
				this.$el.prop( key, value );
			}, this );
		}
	} );

	// Add image attachment button
	bp.Views.AddImageButton = bp.View.extend( {
		tagName: 'div',
		id: 'aw-whats-new-add-image',
		className: 'communityally-activity-add-image-button'
	} );

	// Add file attachment button
	bp.Views.AddFileButton = bp.View.extend( {
		tagName: 'div',
		id: 'aw-whats-new-add-file',
		className: 'communityally-activity-add-file-button'
	} );

	// The content of the activity
	bp.Views.WhatsNew = bp.View.extend( {
		tagName   : 'textarea',
		className : 'bp-suggestions communityally-autogrow-textarea',
		id        : 'whats-new',

		attributes: {
			name         : 'whats-new',
			cols         : '50',
			rows         : '4',
			placeholder  : BP_Nouveau.activity.strings.whatsnewPlaceholder,
			'aria-label' : BP_Nouveau.activity.strings.whatsnewLabel
		},

		initialize: function() {
			if ('group' === this.options.activity.attributes.object) {
				this.$el.attr('data-suggestions-group-id', this.options.activity.attributes.item_id);
			}
			this.on( 'ready', this.adjustContent, this );

			this.options.activity.on( 'change:content', this.resetContent, this );
		},

		adjustContent: function() {

			// First adjust layout
			this.$el.css( {
				height: '50px'
			} );

			// Check for mention
			var	mention = bp.Nouveau.getLinkParams( null, 'r' ) || null;

			if ( ! _.isNull( mention ) ) {
				this.$el.text( '@' + _.escape( mention ) + ' ' );
				this.$el.focus();
			}
		},

		resetContent: function( activity ) {
			if ( _.isUndefined( activity ) ) {
				return;
			}

			this.$el.val( activity.get( 'content' ) );
		}
	} );

	bp.Views.WhatsNewPostIn = bp.View.extend( {
		tagName:   'select',
		id:        'whats-new-post-in',

		attributes: {
			name         : 'whats-new-post-in',
			'aria-label' : BP_Nouveau.activity.strings.whatsnewpostinLabel
		},

		events: {
			change: 'change'
		},

		keys: [],

		initialize: function() {
			this.model = new Backbone.Model();

			this.filters = this.options.filters || {};

			// Build `<option>` elements.
			this.$el.html( _.chain( this.filters ).map( function( filter, value ) {
				return {
					el: $( '<option></option>' ).val( value ).html( filter.text )[0],
					priority: filter.priority || 50
				};
			}, this ).sortBy( 'priority' ).pluck( 'el' ).value() );
		},

		change: function() {
			var filter = this.filters[ this.el.value ];
			if ( filter ) {
				this.model.set( { 'selected': this.el.value, 'placeholder': filter.autocomplete_placeholder } );
			}
		}
	} );

	bp.Views.Item = bp.View.extend( {
		tagName:   'li',
		className: 'bp-activity-object',
		template:  bp.template( 'activity-target-item' ),

		attributes: {
			role: 'checkbox'
		},

		initialize: function() {
			if ( this.model.get( 'selected' ) ) {
				this.el.className += ' selected';
			}
		},

		events: {
			click : 'setObject'
		},

		setObject:function( event ) {
			event.preventDefault();

			if ( true === this.model.get( 'selected' ) ) {
				this.model.clear();
			} else {
				this.model.set( 'selected', true );
			}
		}
	} );

	bp.Views.AutoComplete = bp.View.extend( {
		tagName : 'ul',
		id      : 'whats-new-post-in-box-items',

		events: {
			keyup :  'autoComplete'
		},

		initialize: function() {
			var autocomplete = new bp.Views.ActivityInput( {
				type        : 'text',
				id          : 'activity-autocomplete',
				placeholder : this.options.placeholder || ''
			} ).render();

			this.$el.prepend( $( '<li></li>' ).html( autocomplete.$el ) );

			this.on( 'ready', this.setFocus, this );
			this.collection.on( 'add', this.addItemView, this );
			this.collection.on( 'reset', this.cleanView, this );
		},

		setFocus: function() {
			this.$el.find( '#activity-autocomplete' ).focus();
		},

		addItemView: function( item ) {
			this.views.add( new bp.Views.Item( { model: item } ) );
		},

		autoComplete: function() {
			var search = $( '#activity-autocomplete' ).val();

			// Reset the collection before starting a new search
			this.collection.reset();

			if ( 2 > search.length ) {
				return;
			}

			this.collection.fetch( {
				data: {
					type   : this.options.type,
					search : search,
					nonce  : BP_Nouveau.nonces.activity
				},
				success : _.bind( this.itemFetched, this ),
				error   : _.bind( this.itemFetched, this )
			} );
		},

		itemFetched: function( items ) {
			if ( ! items.length ) {
				this.cleanView();
			}
		},

		cleanView: function() {
			_.each( this.views._views[''], function( view ) {
					view.remove();
			} );
		}
	} );

	bp.Views.FormAvatar = bp.View.extend( {
		tagName   : 'div',
		id        : 'communityally-activity-form-meta',
		className : 'communityally-activity-form-meta',
		template  : bp.template( 'activity-post-form-avatar' ),

		initialize: function() {
			this.model = new Backbone.Model( _.pick( BP_Nouveau.activity.params, [
				'user_id',
				'avatar_url',
				'avatar_width',
				'avatar_height',
				'avatar_alt',
				'user_domain',
				'user_name'
			] ) );

			if ( this.model.has( 'avatar_url' ) ) {
				this.model.set( 'display_avatar', true );
			}
		}
	} );

	bp.Views.FormContent = bp.View.extend( {
		tagName  : 'div',
		id       : 'whats-new-content',
		className: 'whats-new-content',

		initialize: function() {
			this.$el.html( $('<div></div>' ).prop( 'id', 'whats-new-textarea' ).prop('class', 'whats-new-textarea') );
			this.views.set( '#whats-new-textarea', new bp.Views.WhatsNew( { activity: this.options.activity } ) );
		}
	} );

	bp.Views.FormOptions = bp.View.extend( {
		tagName  : 'div',
		id       : 'whats-new-options',
		template : bp.template( 'activity-post-form-options' )
	} );

	bp.Views.CommunityAllyFormOptions = bp.View.extend( {
		tagName  : 'div',
		id       : 'whats-new-options',
		className: 'communityally-activity-new-form-options',

		initialize: function() {
			let editControl = new bp.Views.FormEditControl(),
			submit = new bp.Views.FormSubmit( {
				model:	this.model
			} );

			this.views.set( [ editControl, submit ] );
		}
	} );

	bp.Views.BeforeFormInputs = bp.View.extend( {
		tagName  : 'div',
		template : bp.template( 'activity-before-post-form-inputs' )
	} );

	bp.Views.FormTarget = bp.View.extend( {
		tagName   : 'div',
		id        : 'whats-new-post-in-box',
		className : 'in-profile',

		initialize: function() {
			var select = new bp.Views.WhatsNewPostIn( { filters: BP_Nouveau.activity.params.objects } );
			this.views.add( select );

			select.model.on( 'change', this.attachAutocomplete, this );
			bp.Nouveau.Activity.postForm.ActivityObjects.on( 'change:selected', this.postIn, this );
		},

		attachAutocomplete: function( model ) {
			if ( 0 !== bp.Nouveau.Activity.postForm.ActivityObjects.models.length ) {
				bp.Nouveau.Activity.postForm.ActivityObjects.reset();
			}

			// Clean up views
			_.each( this.views._views[''], function( view ) {
				if ( ! _.isUndefined( view.collection ) ) {
					view.remove();
				}
			} );

			if ( 'profile' !== model.get( 'selected') ) {
				this.views.add( new bp.Views.AutoComplete( {
					collection:   bp.Nouveau.Activity.postForm.ActivityObjects,
					type:         model.get( 'selected' ),
					placeholder : model.get( 'placeholder' )
				} ) );

				// Set the object type
				this.model.set( 'object', model.get( 'selected' ) );

			} else {
				this.model.set( { object: 'user', item_id: 0 } );
			}

			this.updateDisplay();
		},

		postIn: function( model ) {
			if ( _.isUndefined( model.get( 'id' ) ) ) {
				// Reset the item id
				this.model.set( 'item_id', 0 );

				// When the model has been cleared, Attach Autocomplete!
				this.attachAutocomplete( new Backbone.Model( { selected: this.model.get( 'object' ) } ) );
				return;
			}

			// Set the item id for the selected object
			this.model.set( 'item_id', model.get( 'id' ) );

			// Set the view to the selected object
			this.views.set( '#whats-new-post-in-box-items', new bp.Views.Item( { model: model } ) );
		},

		updateDisplay: function() {
			if ( 'user' !== this.model.get( 'object' ) ) {
				this.$el.removeClass( );
			} else if ( ! this.$el.hasClass( 'in-profile' ) ) {
				this.$el.addClass( 'in-profile' );
			}
		}
	} );

	/**
	 * Now build the buttons!
	 * @type {[type]}
	 */
	bp.Views.FormButtons = bp.View.extend( {
		tagName : 'div',
		id      : 'whats-new-actions',

		initialize: function() {
			this.views.add( new bp.View( { tagName: 'ul', id: 'whats-new-buttons' } ) );

			_.each( this.collection.models, function( button ) {
				this.addItemView( button );
			}, this );

			this.collection.on( 'change:active', this.isActive, this );
		},

		addItemView: function( button ) {
			this.views.add( '#whats-new-buttons', new bp.Views.FormButton( { model: button } ) );
		},

		isActive: function( button ) {
			// Clean up views
			_.each( this.views._views[''], function( view, index ) {
				if ( 0 !== index ) {
					view.remove();
				}
			} );

			// Then loop threw all buttons to update their status
			if ( true === button.get( 'active' ) ) {
				_.each( this.views._views['#whats-new-buttons'], function( view ) {
					if ( view.model.get( 'id') !== button.get( 'id' ) ) {
						// Silently update the model
						view.model.set( 'active', false, { silent: true } );

						// Remove the active class
						view.$el.removeClass( 'active' );

						// Trigger an even to let Buttons reset
						// their modifications to the activity model
						this.collection.trigger( 'reset:' + view.model.get( 'id' ), this.model );
					}
				}, this );

				// Tell the active Button to load its content
				this.collection.trigger( 'display:' + button.get( 'id' ), this );

			// Trigger an even to let Buttons reset
			// their modifications to the activity model
			} else {
				this.collection.trigger( 'reset:' + button.get( 'id' ), this.model );
			}
		}
	} );

	bp.Views.FormButton = bp.View.extend( {
		tagName   : 'li',
		className : 'whats-new-button',
		template  : bp.template( 'activity-post-form-buttons' ),

		events: {
			click : 'setActive'
		},

		setActive: function( event ) {
			var isActive = this.model.get( 'active' ) || false;

			// Stop event propagation
			event.preventDefault();

			if ( false === isActive ) {
				this.$el.addClass( 'active' );
				this.model.set( 'active', true );
			} else {
				this.$el.removeClass( 'active' );
				this.model.set( 'active', false );
			}
		}
	} );

	bp.Views.AttachmentItemDisplay = bp.View.extend({
		tagName: 'div',
		className: 'communityally-attachment-display',

		initialize: function() {
			this.model.on('attachment:changed', this.render, this);
		},

		render: function() {
			let status = this.model.get('status');
			this.$el.empty();
			if ('complete' === status) {
				let attachmentType = this.model.get('type'),
					variableName = 'attachment',
					fileName = this.model.get('name');
				if ('image' === attachmentType) {
					let $imageEl = $('<img class="communityally-attachment-image" />');
					$imageEl.attr('src', this.model.get('url'));
					this.$el.append($imageEl);
				} else {
					variableName = 'file-attachment';
					let $fileEl = $('<a target="_blank" class="communityally-attachment-file"></a>');
					$fileEl.text(fileName);
					$fileEl.attr('href', this.model.get('url'));
					this.$el.append($fileEl);
				}

				let ordinal = this.model.get('ordinal'),
					$inputEl = $('<input type="hidden" name="' + variableName + '[' + ordinal + '][name]" />');
				$inputEl.attr('value', fileName);
				this.$el.append($inputEl);

				$inputEl = $('<input type="hidden" name="' + variableName + '[' + ordinal + '][path]" />');
				$inputEl.attr('value', this.model.get('path'));
				this.$el.append($inputEl);
			} else {
				let displayText = this.model.get('name'),
					$displayEl = $('<span class="communityally-attachment-desc"></span>');
				$displayEl.text(displayText);
				this.$el.append($displayEl);

				let $waitEl = $('<div class="communityally-attachment-upload-wait"></div>');
				this.$el.append($waitEl);
			}
			return this;
		}
	});

	bp.Views.AttachmentItemDeleteButton = bp.View.extend({
		tagName: 'div',
		className: 'communityally-attachment-delete-button',

		events: {
			click : 'removeItem',
			clickend : 'removeItem'
		},

		removeItem: function() {
			this.collection.remove(this.model);
		}
	});

	bp.Views.AttachmentItem = bp.View.extend({
		tagName: 'div',
		className: 'communityally-attachment-entry',

		initialize: function() {
			let desc = new bp.Views.AttachmentItemDisplay({
				model: this.model
			}),
			deleteButton = new bp.Views.AttachmentItemDeleteButton({
				model: this.model,
				collection: this.collection
			});
			this.views.set([desc, deleteButton]);
		}
	});

	bp.Views.AttachmentsDisplay = bp.View.extend({
		tagName : 'div',
		collection: null,

		initialize: function() {
			this.id = this.options.id;
			this.className = this.options.className;
			this.collection = this.options.collection;

			this.collection.bind('reset', this.render, this);
			this.collection.bind('add', this.render, this);
			this.collection.bind('remove', this.render, this);
		},
		render: function() {
			let self = this;
			this.views.set([]);

			this.collection.each(function(item) {
				let attachmentView = new bp.Views.AttachmentItem({
					model: item,
					collection: self.collection
				});

				self.views.add(attachmentView);
			});

			return this;
		}
	});

	bp.Views.FormEditControl = bp.View.extend( {
		tagName   : 'div',
		id        : 'whats-new-edit-control',
		className : 'communityally-activity-edit-control',
		initialize: function() {
			let add_image = new bp.Views.AddImageButton(),
				add_file = new bp.Views.AddFileButton();

			this.views.set( [ add_image, add_file ] );
		}
	});

	bp.Views.FormHeader = bp.View.extend( {
		tagName   : 'div',
		id        : 'whats-new-header',
		className : 'whats-new-header communityally-activity-form-header',

		initialize: function() {
			let avatar = new bp.Views.FormAvatar(),
				content = new bp.Views.FormContent(  { activity: this.options.activity } );

			this.views.set( [ avatar, content ] );
		}
	});

	bp.Views.FormSubmit = bp.View.extend( {
		tagName   : 'div',
		id        : 'whats-new-submit',
		className : 'whats-new-submit in-profile',

		initialize: function() {
			var reset = new bp.Views.ActivityInput( {
				type  : 'reset',
				id    : 'aw-whats-new-reset',
				className : 'text-button small communityally-activity-reset-button',
				value : BP_Nouveau.activity.strings.cancelButton
			} );

			var submit = new bp.Views.ActivityInput( {
				type  : 'submit',
				id    : 'aw-whats-new-submit',
				className : 'button communityally-button communityally-activity-submit-button',
				name  : 'aw-whats-new-submit',
				value : BP_Nouveau.activity.strings.postUpdateButton
			} );

			this.views.set( [ reset, submit ] );

			this.model.on( 'change:object', this.updateDisplay, this );
		},

		updateDisplay: function( model ) {
			if ( _.isUndefined( model ) ) {
				return;
			}

			if ( 'user' !== model.get( 'object' ) ) {
				this.$el.removeClass( 'in-profile' );
			} else if ( ! this.$el.hasClass( 'in-profile' ) ) {
				this.$el.addClass( 'in-profile' );
			}
		}
	} );

	bp.Views.PostForm = bp.View.extend( {
		tagName   : 'form',
		className : 'activity-form',
		id        : 'whats-new-form',

		attributes: {
			name   : 'whats-new-form',
			method : 'post'
		},

		events: {
			'reset'            : 'resetForm',
			'submit'           : 'postUpdate',
			'keydown'          : 'postUpdate'
		},

		initialize: function() {
			this.model = new bp.Models.Activity( _.pick(
				BP_Nouveau.activity.params,
				['user_id', 'item_id', 'object' ]
			) );

			this.options.backcompat = BP_Nouveau.activity.params.backcompat;
			var staticViews = [
				new bp.Views.FormHeader( { activity: this.model } ),
				new bp.Views.AttachmentsDisplay( { id: 'whats-new-attachments', className: 'whats-new-attachments communityally-activity-attachments',
					collection: this.model.attachments, model: this.model } ),
				new bp.Views.AttachmentsDisplay( { id: 'whats-new-file-attachments', className: 'whats-new-file-attachments communityally-activity-file-attachments',
					collection: this.model.fileAttachments, model: this.model } ),
				new bp.Views.CommunityAllyFormOptions( { model: this.model } )
			];

			// Backcompat to take the `bp_before_activity_post_form` action in account.
			if ( true === this.options.backcompat.before_post_form ) {
				staticViews.unshift( new bp.Views.BeforeFormInputs() );
			}

			// Clone the model to set the resetted one
			this.resetModel = this.model.clone();

			this.views.set( staticViews );
			this.model.on( 'change:errors', this.displayFeedback, this );
			// Remove feedback.
			this.cleanFeedback();

			this.$el.addClass( 'activity-form-expanded' );
			let self = this;
			this.$el.ready(function() {
				self.imageAttachmentUploader = createNewCommunityAllyUploader('aw-whats-new-add-image', self.model.attachments, 'image');
				self.fileAttachmentUploader = createNewCommunityAllyUploader('aw-whats-new-add-file', self.model.fileAttachments, 'file');
			});
		},

		resetForm: function() {
			var self = this, indexStaticViews = self.options.backcompat.before_post_form ? 4 : 3;

			_.each( this.views._views[''], function( view, index ) {
				if ( index > indexStaticViews ) {
					view.remove();
				}
			} );

			$( '#whats-new' ).css( {
				height : '50px'
			} );

			this.$el.removeClass( 'activity-form-expanded' );

			// Reset the model
			this.model.clear();
			this.model.set( this.resetModel.attributes );
			this.model.attachments.reset();
			this.model.fileAttachments.reset();
		},

		cleanFeedback: function() {
			this.model.unset( 'errors', { silent: true } );
			_.each( this.views._views[''], function( view ) {
				if ( 'message' === view.$el.prop( 'id' ) ) {
					view.remove();
				}
			} );
		},

		displayFeedback: function( model ) {
			if ( _.isUndefined( this.model.get( 'errors' ) ) ) {
				this.cleanFeedback();
			} else {
				this.views.add( new bp.Views.activityFeedback( model.get( 'errors' ) ) );
			}
		},

		postUpdate: function( event ) {
			var self = this,
			    meta = {},
				communityally_wait_overlay = $('#communityally-wait-overlay');

			if ( event ) {
				if ( 'keydown' === event.type && ( 13 !== event.keyCode || ! event.ctrlKey ) ) {
					return event;
				}

				event.preventDefault();
			}

			communityally_wait_overlay.show();
			if (!this.model.isUploadCompleted()) {
				setTimeout(function() { self.postUpdate(event); }, 1000);
				return;
			}

			// Set the content and meta
			_.each( this.$el.serializeArray(), function( pair ) {
				pair.name = pair.name.replace( '[]', '' );
				if ( 'whats-new' === pair.name ) {
					self.model.set( 'content', pair.value );
				} else if ( -1 === _.indexOf( ['aw-whats-new-submit', 'whats-new-post-in'], pair.name ) ) {
					if ( _.isUndefined( meta[ pair.name ] ) ) {
						meta[ pair.name ] = pair.value;
					} else {
						if ( ! _.isArray( meta[ pair.name ] ) ) {
							meta[ pair.name ] = [ meta[ pair.name ] ];
						}

						meta[ pair.name ].push( pair.value );
					}
				}
			} );

			// Silently add meta
			this.model.set( meta, { silent: true } );

			var data = {
				'_wpnonce_post_update': BP_Nouveau.activity.params.post_nonce
			};

			// Add the Akismet nonce if it exists.
			if ( $('#_bp_as_nonce').val() ) {
				data._bp_as_nonce = $('#_bp_as_nonce').val();
			}

			bp.ajax.post( 'post_update', _.extend( data, this.model.attributes ) ).done( function( response ) {
				var store       = bp.Nouveau.getStorage( 'bp-activity' ),
					searchTerms = $( '[data-bp-search="activity"] input[type="search"]' ).val(), matches = {},
					toPrepend = false;

				// Look for matches if the stream displays search results.
				if ( searchTerms ) {
					searchTerms = new RegExp( searchTerms, 'im' );
					matches = response.activity.match( searchTerms );
				}

				/**
				 * Before injecting the activity into the stream, we need to check the filter
				 * and search terms are consistent with it when posting from a single item or
				 * from the Activity directory.
				 */
				if ( ( ! searchTerms || matches ) ) {
					toPrepend = ! store.filter || 0 === parseInt( store.filter, 10 ) || 'activity_update' === store.filter;
				}

				/**
				 * In the Activity directory, we also need to check the active scope.
				 * eg: An update posted in a private group should only show when the
				 * "My Groups" tab is active.
				 */
				if ( toPrepend && response.is_directory ) {
					toPrepend = ( 'all' === store.scope && ( 'user' === self.model.get( 'object' ) || false === response.is_private ) ) || ( self.model.get( 'object' ) + 's'  === store.scope );
				}

				// Reset the form
				self.resetForm();

				// Display a successful feedback if the acticity is not consistent with the displayed stream.
				if ( ! toPrepend ) {
					self.views.add( new bp.Views.activityFeedback( { value: response.message, type: 'updated' } ) );

				// Inject the activity into the stream only if it hasn't been done already (HeartBeat).
				} else if ( ! $( '#activity-' + response.id  ).length ) {

					// It's the very first activity, let's make sure the container can welcome it!
					if ( ! $( '#activity-stream ul.activity-list').length ) {
						$( '#activity-stream' ).html( $( '<ul></ul>').addClass( 'activity-list item-list bp-list' ) );
					}

					// Prepend the activity.
					let content_container_selector = '#activity-stream ul.activity-list';
					if ('group' === self.model.get( 'object' )) {
						content_container_selector = '.communityally-group-activity-list-' + self.model.get( 'item_id' );

						// remove any "no activity" message
						$(content_container_selector).find('.bp-feedback').remove();
					}
					bp.Nouveau.inject( content_container_selector, response.activity, 'prepend' );
				}
				communityally_wait_overlay.hide();
			} ).fail( function( response ) {
				communityally_wait_overlay.hide();
				self.model.set( 'errors', { type: 'error', value: response.message } );
			} );
		}
	} );

	bp.Nouveau.Activity.postForm.start();

} )( window.bp, jQuery );
