/* jshint browser: true */
/* global BP_Nouveau, plupload */
/* @since 3.0.0 */
/* @version 8.0.0 */
window.bp = window.bp || {};

( function( bp, $ ) {
	// Bail if not set
	if ( typeof BP_Nouveau === 'undefined' ) {
		return;
	}

	bp.Nouveau = bp.Nouveau || {};

	function serialize_value_in_container($container) {
		var values = [];
		$container.find('input[type="hidden"],input[type="text"],textarea').each(function(index, elem) {
			var $elem = $(elem),
				attr = $elem.attr('communityally-param');
			if (typeof attr !== typeof undefined && attr !== false) {
				values.push({ name : attr, value : $elem.val() });
			}
		});
		$container.find('input[type="radio"]:checked').each(function(index, elem) {
			var $elem = $(elem),
				attr = $elem.attr('name');
			if (typeof attr !== typeof undefined && attr !== false) {
				values.push({ name : attr, value : $elem.attr('value') });
			}
		});
		$container.find('input[type="checkbox"]:checked').each(function(index, elem) {
			var $elem = $(elem),
				attr = $elem.attr('communityally-param');
			if (typeof attr !== typeof undefined && attr !== false) {
				values.push({ name : attr, value : $elem.attr('value') });
			}
		});
		$container.find('select').each(function(index, elem) {
			var $elem = $(elem),
				attr = $elem.attr('communityally-param');
			if (typeof attr !== typeof undefined && attr !== false) {
				values.push({ name : attr, value : $elem.children(':selected').val() });
			}
		});
		return values;
	}
	// <editor-fold defaultstate="collapsed" desc="Copy Discussion shortcode">
	var $hidden_copy_element = false;
	function copy_text_to_clipboard($parent_element, text_to_copy) {
		var current_focus_element = document.activeElement,
			is_success = false;
		if (!$hidden_copy_element) {
			$hidden_copy_element = $('<textarea style="position:absolute;top:0;left:-99999px;width:0;height:0"></textarea>');
		}
		$hidden_copy_element.show();
		$parent_element.append($hidden_copy_element);

		$hidden_copy_element.val(text_to_copy);

		// select the hidden element content
		$hidden_copy_element[0].focus();
		$hidden_copy_element[0].setSelectionRange(0, text_to_copy.length);

		// copy the selection
		try {
			is_success = document.execCommand("copy");
		} catch(e) {
		}
		// restore original focus
		if (current_focus_element && typeof current_focus_element.focus === "function") {
			current_focus_element.focus();
		}

		$hidden_copy_element.val('').hide();
		return is_success;
	}
	$(document).on('click touchend', '[communityally-discussion-shortcode-copy-button]', function() {
		var $this = $(this),
			$parent = $this.parents('.communityally-discussion-entry-shortcode'),
			group_id = $this.attr('communityally-discussion-shortcode-copy-button'),
			val = '[communityally_group group_id="' + group_id + '"]';
		if (copy_text_to_clipboard($parent, val)) {
			$parent.find('.communityally-discussion-shortcode-copy-success').show();
			setTimeout(function() {
				$parent.find('.communityally-discussion-shortcode-copy-success').hide();
			}, 2000);
		} else {
			$parent.find('.communityally-discussion-shortcode-copy-fail').show();
			setTimeout(function() {
				$parent.find('.communityally-discussion-shortcode-copy-fail').hide();
			}, 2000);
		}
	});
	// </editor-fold>
	bp.Nouveau.Discussion = {

		start: function () {
			this.addListeners();
		},

		addListeners: function () {
			// create new group
			$(document).on('click touchend', '.communityally-discussion-add-group-button', this.showAddGroupModal);
			$(document).on('click touchend', '.communityally-modal-close', function () {
				let target_modal_to_close = $(this).attr('communityally-modal-close');
				$( '#' + target_modal_to_close ).hide();
			});
			$(document).on('click touchend', '#communityally-add-child-group-submit', this.addChildGroupAjax);
			$(document).on('click touchend', '[communityally-edit-discussion-id]', this.showEditDiscussionModal);
			$(document).on('click touchend', '[communityally-discussion-edit-modal-submit]', this.editDiscussionAjax);
			$(document).on('click touchend', '[communityally-delete-discussion-id]', this.deleteDiscussion);
		},

		showEditDiscussionModal: function () {
			let group_id = $(this).attr('communityally-edit-discussion-id');
			$('#communityally-edit-child-group-modal-' + group_id).show();
		},
		editDiscussionAjax: function () {
			let group_id = $(this).attr('communityally-discussion-edit-modal-submit'),
				$modal = $('#communityally-edit-child-group-modal-' + group_id),
				details = serialize_value_in_container($modal),
				data = {
					action: 'groups_edit_child_group',
					group_id: group_id,
					group_settings: JSON.stringify(details)
			}, response = '';

			$('#communityally-wait-overlay').show();
			response = bp.Nouveau.ajax( data, 'groups');
			response.done( function () {
				let data = response.responseJSON.data;
				if ( !response.responseJSON.success ) {
					alert('Error occurred when updating discussion: ' + data.feedback);
				} else {
					// update the discussion-list display
					$('.communityally-discussion-content-topics-container').html(data['code']);
				}
			});
			response.always( function () {
				$('#communityally-wait-overlay').hide();
			});
		},
		deleteDiscussion: function () {
			let data, response,
				conf = confirm('Deleting this discussion will completely remove ALL content associated with it. There is no way back. Are you sure you want to delete it?');
			if ( conf !== true) {
				return false;
			}

			data = {
				action: 'groups_delete_child_group',
				group_id: $(this).attr('communityally-delete-discussion-id')
			}

			$('#communityally-wait-overlay').show();
			response = bp.Nouveau.ajax( data, 'groups');
			response.done( function () {
				let data = response.responseJSON.data;
				if ( !response.responseJSON.success ) {
					alert('Error occurred when deleting discussion: ' + data.feedback);
				} else {
					// update the discussion-list display
					$('.communityally-discussion-content-topics-container').html(data['code']);
				}
			});
			response.always( function () {
				$('#communityally-wait-overlay').hide();
			});
		},
		/**
		 *  make add child-group modal visible
		 * */
		showAddGroupModal: function () {
			$( '#communityally-add-child-group-modal' ).show();
		},

		/**
		 * gather information and send request to create group
		 * */
		addChildGroupAjax: function () {
			let data = {
				action: 'groups_add_child_group',
				group_title: $('[communityally-param="title"]').val(),
				group_description: $('[communityally-param="group-description"]').val(),
			}, response = '';

			$('#communityally-wait-overlay').show();
			response = bp.Nouveau.ajax( data, 'groups');
			response.done( function () {
				let data = response.responseJSON.data;
				if ( !response.responseJSON.success ) {
					alert('Error occurred when creating discussion: ' + data.feedback);
				} else {
					$('.communityally-discussion-content-topics-container').html(data['code']);
					// clear form after successful submission
					$('#communityally-add-child-group-modal [communityally-param]').each( function (index, elem) {
						$(elem).val('');
					});
				}
			});

			response.always( function () {
				$( '#communityally-add-child-group-modal' ).hide();
				$( '#communityally-wait-overlay' ).hide();
			});
		},
	};

	bp.Nouveau.Discussion.start();

})(window.bp, jQuery)